/*
Fri Aug 15 16:29:47 EDT 1997

                       ***   SAMPLE USAGE    ***
                    For Harwell I/O package iohb.c.

           National Institute of Standards and Technology, MD.
                             K.A. Remington

++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
                                NOTICE

 Permission to use, copy, modify, and distribute this software and
 its documentation for any purpose and without fee is hereby granted
 provided that the above copyright notice appear in all copies and
 that both the copyright notice and this permission notice appear in
 supporting documentation.

 Neither the Author nor the Institution (National Institute of Standards
 and Technology) make any representations about the suitability of this 
 software for any purpose. This software is provided "as is" without 
 expressed or implied warranty.
++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*/
#include<stdlib.h>
#include<stdio.h>
#include "iohb.h"

int netsl(char *,...);

int main(int argc, char *argv[])
{
   char temp_str[10];
   int operation, pack_type, status;
   int M, N, nonzeros, Nrhs;
   int *colptr, *rowind;
   double *val;
   double *rhs;
   double* sol_vector;
   char *Type;
   int i;

   if (argc < 2)
   {
     fprintf(stderr,"Usage: %s HBfile \n", argv[0]);
     exit(-1);
   }

/* Get information about the array stored in the file specified in the  */
/* argument list:                                                       */

   fprintf(stderr,"Reading matrix info from %s...\n",argv[1]);
   readHB_info(argv[1], &M, &N, &nonzeros, &Type, &Nrhs);

   fprintf(stderr,"Matrix in file %s is %d x %d, with %d nonzeros with type %s;\n",
                         argv[1], M, N, nonzeros, Type);
   fprintf(stderr,"%d right-hand-side(s) available.\n",Nrhs);

/* Read the matrix information, generating the associated storage arrays  */ 
   fprintf(stderr,"Reading the matrix from %s...\n",argv[1]);
   readHB_newmat_double(argv[1], &M, &N, &nonzeros, &colptr, &rowind, &val);
   
  printf("%d %d %d\n", M, N, nonzeros);
  
/* If a rhs is specified in the file, read one, generating the associate storage */
   if (Nrhs > 0) {
      fprintf(stderr,"Reading right-hand-side vector(s) from %s...\n",argv[1]);
      readHB_newaux_double(argv[1], 'F', &rhs);
   }
   else{

     rhs = (double*)malloc(sizeof(double)*M);

     for(i=0; i<M; i++)
       rhs[i] = 1.0;
   }

   sol_vector = (double*)malloc(sizeof(double)*M);

  printf("Choose operation (iterative=%d, direct=%d): ",
    1,2);
  scanf("%d",&operation);

  if (operation==1) {
    double rtol; int maxit,its;

    printf("\n\n Choose package type(1. PETSC 2. AZTEC) : ");
    scanf("%d", &pack_type);
    if(pack_type == 1)
      strcpy(temp_str, "PETSC");
    else
      strcpy(temp_str, "AZTEC");

    printf("Relative tolerance : "); scanf("%lf",&rtol); printf("\n");
    printf("Maximum #iterations : "); scanf("%d",&maxit); printf("\n");

    status = netsl("sparse_iterative_solve()", temp_str,
			M, nonzeros,
                val, rowind, colptr, rhs, &rtol,
                &maxit, sol_vector, &its);

#ifdef DEBUG
  printf("iteratons: %d\n", its);
#endif

    }
    else if (operation==2) {
      double pivot;
      int perm;

      printf("\n\n Choose package type(1. MA28 2.SUPERLU):  ");
      fflush(stdin);
      scanf("%d", &pack_type);
      if(pack_type == 1){
        strcpy(temp_str, "MA28");
 
        printf("Pivot parameter 0--1 (0=positional, 1=numerical) : ");
        scanf("%lf",&pivot); printf("\n");
      }
      else{
        strcpy(temp_str, "SUPERLU");
        printf("Permutation (0=nat, 1=md(At*A), 2=md(At+A)) : ");
        scanf("%d",&perm); printf("\n");
      }

      status = netsl("sparse_direct_solve()", temp_str,  M, nonzeros,
              val, rowind, colptr, rhs, &pivot,
              &perm, sol_vector);
    }
    if(status < 0){
      netslerr(status);
      printf("erro:%s\n",netsolveErrorMessage(status));
    }
    else
      printf("completed\n");
#ifdef DEBUG
  ierr = print_result(sol_vector, local_size);
#endif

   return 0;
}
